/**
 * Author: Andrei Heidelbacher
 * Time complexity: O(N)
 * Space complexity: O(N)
 * 100 points
 */

#include <fstream>
#include <vector>
#include <algorithm>
#include <unordered_map>

using namespace std;

const char IN_FILE[] = "sushi.in";
const char OUT_FILE[] = "sushi.out";
const int NIL = -1;
const int oo = 0x3f3f3f3f;

typedef vector<int>::const_iterator Edge;

struct Query {
 int x, y, t;
};

vector<vector<int>> Tree;
int V, Root;
vector<int> Euler;
unordered_map<int64_t, int> Position;
int Q;
vector<Query> Queries;
vector<int> Values;
vector<int> MinT;

inline int edgeIndex(const int x, const int y) {
  return 1LL * x * V + y;
}

void eulerDfs(const int x, const int father = NIL) {
  Euler.push_back(x);
  for (Edge y = Tree[x].begin(); y != Tree[x].end(); ++y) {
    if (*y != father) {
      Position[edgeIndex(x, *y)] = int(Euler.size()) - 1;
      eulerDfs(*y, x);
      Position[edgeIndex(*y, x)] = int(Euler.size()) - 1;
      Euler.push_back(x);
    }
  }
}

void buildValues() {
  eulerDfs(0);
  const int eulerSize = int(Euler.size());
  Values = vector<int>(eulerSize, -oo);
  for (int q = 0; q < Q; ++q) {
    const int p = Position[edgeIndex(Queries[q].x, Queries[q].y)];
    Values[p] = max(Values[p], Queries[q].t - p);
  }
}

void solveLeft() {
  vector<bool> visited = vector<bool>(V, false);
  int prefixMaxValue = -oo;
  for (int i = 0; i < int(Euler.size()); ++i) {
    const int x = Euler[i];
    prefixMaxValue = max(prefixMaxValue, Values[i]);
    if (!visited[x]) {
      MinT[x] = max(MinT[x], prefixMaxValue + i);
      visited[x] = true;
    }
  }
}

void solveRight() {
  vector<bool> visited = vector<bool>(V, false);
  int suffixMaxValue = -oo;
  for (int i = int(Euler.size()) - 1; i >= 0; --i) {
    const int x = Euler[i];
    if (!visited[x]) {
      MinT[x] = max(MinT[x], suffixMaxValue + i + int(Euler.size()) - 1);
      visited[x] = true;
    }
    suffixMaxValue = max(suffixMaxValue, Values[i]);
  }
}

void solveMiddle() {
  vector<pair<int, int>> stack;
  vector<bool> visited = vector<bool>(V, false);
  for (int i = 0; i < int(Euler.size()); ++i) {
    const int x = Euler[i];
    if (!visited[x]) {
      visited[x] = true;
      stack.push_back(make_pair(x, Values[i]));
    } else {
      int maxValue = Values[i];
      while (!stack.empty() && stack.back().first != x) {
        maxValue = max(maxValue, stack.back().second);
        stack.pop_back();
      }
      MinT[x] = max(MinT[x], i + maxValue);
      maxValue = max(maxValue, stack.back().second);
      stack.pop_back();
      stack.push_back(make_pair(x, maxValue));
    }
  }
}

void solve() {
  buildValues();
  solveLeft();
  solveRight();
  solveMiddle();
}

void read() {
  ifstream in(IN_FILE);
  in >> V >> Q;
  Tree = vector<vector<int>>(V);
  for (int x = 0; x < V; ++x) {
    int degree;
    in >> degree;
    Tree[x] = vector<int>(degree);
    for (int i = 0; i < degree; ++i) {
      in >> Tree[x][i];
      --Tree[x][i];
    }
  }
  Root = 0;
  Queries = vector<Query>(Q);
  for (int q = 0; q < Q; ++q) {
    in >> Queries[q].x >> Queries[q].y >> Queries[q].t;
    --Queries[q].x;
    --Queries[q].y;
    Queries[q].y = Tree[Queries[q].x][Queries[q].y];
  }
  MinT = vector<int>(V, 0);
  in.close();
}

void write() {
  ofstream out(OUT_FILE);
  for (int x = 0; x < V; ++x)
    out << MinT[x] << ((x == V - 1) ? "\n" : " ");
  out.close();
}

int main() {
  read();
  solve();
  write();
  return 0;
}
