/**
 * Author: Andrei Heidelbacher
 * Time complexity: O(N)
 * Space complexity: O(N)
 */

#include <fstream>
#include <vector>
#include <algorithm>

using namespace std;

const char IN_FILE[] = "sushi.in";
const char OUT_FILE[] = "sushi.out";
const int NIL = -1;
const int oo = 0x3f3f3f3f;

struct Edge {
  int vertex, index;
};

struct Query {
  int index, t;
};

vector<vector<Edge>> Tree;
int V, Root;
vector<int> Euler, First, Last, Position;
int Q;
vector<Query> Queries;
vector<int> Values, MinT;

void eulerDfs(const int x, const int father = NIL) {
  First[x] = Last[x] = int(Euler.size());
  Euler.push_back(x);
  for (int e = (father != NIL ? 1 : 0); e < int(Tree[x].size()); ++e) {
    const int y = Tree[x][e].vertex;
    if (y != father) {
      Position[Tree[x][e].index] = int(Euler.size()) - 1;
      eulerDfs(y, x);
      Last[x] = int(Euler.size());
      Euler.push_back(x);
    }
  }
  if (father != NIL)
    Position[Tree[x][0].index] = int(Euler.size()) - 1;
}

void buildEuler() {
  First = Last = vector<int>(V, -1);
  eulerDfs(0);
}

void buildValues() {
  const int eulerSize = int(Euler.size());
  Values = vector<int>(eulerSize, -oo);
  for (int q = 0; q < Q; ++q) {
    const int p = Position[Queries[q].index];
    Values[p] = max(Values[p], Queries[q].t - p);
  }
}

void solveLeft() {
  int prefixMaxValue = -oo;
  for (int i = 0; i < int(Euler.size()); ++i) {
    const int x = Euler[i];
    prefixMaxValue = max(prefixMaxValue, Values[i]);
    if (i == First[x])
      MinT[x] = max(MinT[x], prefixMaxValue + i);
  }
}

void solveRight() {
  int suffixMaxValue = -oo;
  for (int i = int(Euler.size()) - 1; i >= 0; --i) {
    const int x = Euler[i];
    if (i == Last[x])
      MinT[x] = max(MinT[x], suffixMaxValue + First[x] + int(Euler.size()) - 1);
    suffixMaxValue = max(suffixMaxValue, Values[i]);
  }
}

void solveMiddle() {
  vector<pair<int, int>> stack;
  for (int i = 0; i < int(Euler.size()); ++i) {
    const int x = Euler[i];
    if (i == First[x]) {
      stack.push_back(make_pair(x, Values[i]));
    } else {
      int maxValue = Values[i];
      while (!stack.empty() && stack.back().first != x) {
        maxValue = max(maxValue, stack.back().second);
        stack.pop_back();
      }
      MinT[x] = max(MinT[x], i + maxValue);
      maxValue = max(maxValue, stack.back().second);
      stack.pop_back();
      stack.push_back(make_pair(x, maxValue));
    }
  }
}

void solve() {
  buildEuler();
  buildValues();
  solveLeft();
  solveRight();
  solveMiddle();
}

void read() {
  ifstream in(IN_FILE);
  in >> V >> Q;
  Tree = vector<vector<Edge>>(V);
  Root = 0;
  int edgeIndex = 0;
  for (int x = 0; x < V; ++x) {
    int degree;
    in >> degree;
    Tree[x] = vector<Edge>(degree);
    for (int i = 0; i < degree; ++i) {
      in >> Tree[x][i].vertex;
      --Tree[x][i].vertex;
      Tree[x][i].index = edgeIndex++;
    }
  }
  Position = vector<int>(edgeIndex, -1);
  Queries = vector<Query>(Q);
  for (int q = 0; q < Q; ++q) {
    int x, i;
    in >> x >> i >> Queries[q].t;
    --x;
    --i;
    Queries[q].index = Tree[x][i].index;
  }
  MinT = vector<int>(V, 0);
  in.close();
}

void write() {
  ofstream out(OUT_FILE);
  for (int x = 0; x < V; ++x)
    out << MinT[x] << ((x == V - 1) ? "\n" : " ");
  out.close();
}

int main() {
  read();
  solve();
  write();
  return 0;
}
