/**
 * Author: Andrei Heidelbacher
 * Time complexity: O(N^2)
 * Space complexity: O(N)
 * 40 points
 */

#include <fstream>
#include <vector>
#include <algorithm>
#include <unordered_map>

using namespace std;

const char IN_FILE[] = "sushi.in";
const char OUT_FILE[] = "sushi.out";
const int NIL = -1;
const int oo = 0x3f3f3f3f;

typedef vector<int>::const_iterator Edge;

struct Query {
 int x, y, t;
};

vector<vector<int>> Tree;
int V, Root;
vector<int> Euler;
unordered_map<int64_t, int> Position;
int Q;
vector<Query> Queries;
vector<int> Values;

inline int edgeIndex(const int x, const int y) {
  return 1LL * x * V + y;
}

void eulerDfs(const int x, const int father = NIL) {
  Euler.push_back(x);
  for (Edge y = Tree[x].begin(); y != Tree[x].end(); ++y) {
    if (*y != father) {
      Position[edgeIndex(x, *y)] = int(Euler.size()) - 1;
      eulerDfs(*y, x);
      Position[edgeIndex(*y, x)] = int(Euler.size()) - 1;
      Euler.push_back(x);
    }
  }
}

void buildValues() {
  eulerDfs(0);
  const int eulerSize = int(Euler.size());
  Values = vector<int>(eulerSize, -oo);
  for (int q = 0; q < Q; ++q) {
    const int p = Position[edgeIndex(Queries[q].x, Queries[q].y)];
    Values[p] = max(Values[p], Queries[q].t - p);
  }
  for (int i = 1; i < eulerSize; ++i) {
    Euler.push_back(Euler[i]);
    Values.push_back(Values[i] - eulerSize + 1);
  }
}

void solve() {
  buildValues();
}

void read() {
  ifstream in(IN_FILE);
  in >> V >> Q;
  Tree = vector<vector<int>>(V);
  for (int x = 0; x < V; ++x) {
    int degree;
    in >> degree;
    Tree[x] = vector<int>(degree);
    for (int i = 0; i < degree; ++i) {
      in >> Tree[x][i];
      --Tree[x][i];
    }
  }
  Root = 0;
  Queries = vector<Query>(Q);
  for (int q = 0; q < Q; ++q) {
    in >> Queries[q].x >> Queries[q].y >> Queries[q].t;
    --Queries[q].x;
    --Queries[q].y;
    Queries[q].y = Tree[Queries[q].x][Queries[q].y];
  }
  in.close();
}

void write() {
  ofstream out(OUT_FILE);
  for (int x = 0; x < V; ++x) {
    int minT = 0;
    for (int i = 0, maxValue = -oo; i < int(Euler.size()); ++i) {
      maxValue = max(maxValue, Values[i]);
      if (Euler[i] == x) {
        minT = max(minT, i + maxValue);
        maxValue = -oo;
      }
    }
    out << minT << ((x == V - 1) ? "\n" : " ");
  }
  out.close();
}

int main() {
  read();
  solve();
  write();
  return 0;
}
