/**
 * Author: Andrei Heidelbacher
 * Time complexity: O(N)
 * Space complexity: O(N)
 */

#include <fstream>
#include <vector>
#include <algorithm>

using namespace std;

typedef vector<int>::const_iterator edge;

const char IN_FILE[] = "arbore.in";
const char OUT_FILE[] = "arbore.out";
const int NIL = -1;
const int MOD = 1000000007;

vector<vector<int>> Tree;
int V, Root;
vector<vector<int>> ValueDP, CountDP;

inline void multiply(int &a, const int b) {
  a = (1LL * a * b) % MOD;
}

void dfs(const int x, const int father = NIL) {
  ValueDP[x][0] = 1;
  ValueDP[x][1] = 0;
  CountDP[x][0] = CountDP[x][1] = 1;

  for (edge y = Tree[x].begin(); y != Tree[x].end(); ++y) {
    if (*y != father) {
      dfs(*y, x);

      ValueDP[x][0] += max(ValueDP[*y][0] - 1, ValueDP[*y][1]);
      if (ValueDP[*y][0] - 1 > ValueDP[*y][1])
        multiply(CountDP[x][0], CountDP[*y][0]);
      else if (ValueDP[*y][1] > ValueDP[*y][0] - 1)
        multiply(CountDP[x][0], CountDP[*y][1]);
      else
        multiply(CountDP[x][0], CountDP[*y][0] + CountDP[*y][1]);

      ValueDP[x][1] += max(ValueDP[*y][0], ValueDP[*y][1]);
      if (ValueDP[*y][0] > ValueDP[*y][1])
        multiply(CountDP[x][1], CountDP[*y][0]);
      else if (ValueDP[*y][1] > ValueDP[*y][0])
        multiply(CountDP[x][1], CountDP[*y][1]);
      else
        multiply(CountDP[x][1], CountDP[*y][0] + CountDP[*y][1]);
    }
  }
}

void solve() {
  ValueDP = CountDP = vector<vector<int>>(V, vector<int>(2));
  dfs(Root);
}

inline void addEdge(const int x, const int y) {
  Tree[x].push_back(y);
  Tree[y].push_back(x);
}

void read() {
  ifstream in(IN_FILE);
  in >> V;
  Tree = vector<vector<int>>(V, vector<int>());
  for (int i = 0; i < V - 1; ++i) {
    int x, y;
    in >> x >> y;
    addEdge(x - 1, y - 1);
  }
  Root = 0;
  in.close();
}

void write() {
  ofstream out(OUT_FILE);
  int value = max(ValueDP[Root][0], ValueDP[Root][1]), count = 0;
  if (ValueDP[Root][0] > ValueDP[Root][1])
    count = CountDP[Root][0];
  else if (ValueDP[Root][1] > ValueDP[Root][0])
    count = CountDP[Root][1];
  else
    count = (CountDP[Root][0] + CountDP[Root][1]) % MOD;
  out << value << " " << count << "\n";
  out.close();
}

int main() {
  read();
  solve();
  write();
  return 0;
}
